@@ -112,12 +112,12 @@ static int node_symlink(struct udev_device *dev, const char *node, const char *s
                 mac_selinux_create_file_clear();
         } while (err == -ENOENT);
         if (err != 0) {
-                log_error("symlink '%s' '%s' failed: %m", target, slink_tmp);
+                log_error_errno(errno, "symlink '%s' '%s' failed: %m", target, slink_tmp);
                 goto exit;
         }
         err = rename(slink_tmp, slink);
         if (err != 0) {
-                log_error("rename '%s' '%s' failed: %m", slink_tmp, slink);
+                log_error_errno(errno, "rename '%s' '%s' failed: %m", slink_tmp, slink);
                 unlink(slink_tmp);
         }
 exit:
@@ -186,7 +186,7 @@ static void link_update(struct udev_device *dev, const char *slink, bool add) {
         char buf[UTIL_PATH_SIZE];
 
         util_path_encode(slink + strlen("/dev"), name_enc, sizeof(name_enc));
-        strscpyl(dirname, sizeof(dirname), "/run/udev/links/", name_enc, NULL);
+        strscpyl(dirname, sizeof(dirname), UDEV_ROOT_RUN "/udev/links/", name_enc, NULL);
         strscpyl(filename, sizeof(filename), dirname, "/", udev_device_get_id_filename(dev), NULL);
 
         if (!add && unlink(filename) == 0)
@@ -264,7 +264,7 @@ static int node_permissions_apply(struct udev_device *dev, bool apply,
 
         if (lstat(devnode, &stats) != 0) {
                 err = -errno;
-                log_debug("can not stat() node '%s' (%m)", devnode);
+                log_debug_errno(errno, "can not stat() node '%s' (%m)", devnode);
                 goto out;
         }
 
@@ -283,10 +283,10 @@ static int node_permissions_apply(struct udev_device *dev, bool apply,
                         log_debug("set permissions %s, %#o, uid=%u, gid=%u", devnode, mode, uid, gid);
                         err = chmod(devnode, mode);
                         if (err < 0)
-                                log_warning("setting mode of %s to %#o failed: %m", devnode, mode);
+                                log_warning_errno(errno, "setting mode of %s to %#o failed: %m", devnode, mode);
                         err = chown(devnode, uid, gid);
                         if (err < 0)
-                                log_warning("setting owner of %s to uid=%u, gid=%u failed: %m", devnode, uid, gid);
+                                log_warning_errno(errno, "setting owner of %s to uid=%u, gid=%u failed: %m", devnode, uid, gid);
                 } else {
                         log_debug("preserve permissions %s, %#o, uid=%u, gid=%u", devnode, mode, uid, gid);
                 }
@@ -304,7 +304,7 @@ static int node_permissions_apply(struct udev_device *dev, bool apply,
 
                                 r = mac_selinux_apply(devnode, label);
                                 if (r < 0)
-                                        log_error("SECLABEL: failed to set SELinux label '%s': %s", label, strerror(-r));
+                                        log_error_errno(r, "SECLABEL: failed to set SELinux label '%s': %m", label);
                                 else
                                         log_debug("SECLABEL: set SELinux label '%s'", label);
 
@@ -313,7 +313,7 @@ static int node_permissions_apply(struct udev_device *dev, bool apply,
 
                                 r = mac_smack_apply(devnode, label);
                                 if (r < 0)
-                                        log_error("SECLABEL: failed to set SMACK label '%s': %s", label, strerror(-r));
+                                        log_error_errno(r, "SECLABEL: failed to set SMACK label '%s': %m", label);
                                 else
                                         log_debug("SECLABEL: set SMACK label '%s'", label);
 
@@ -323,7 +323,7 @@ static int node_permissions_apply(struct udev_device *dev, bool apply,
 
                 /* set the defaults */
                 if (!selinux)
-                        label_fix(devnode, true, false);
+                        mac_selinux_fix(devnode, true, false);
                 if (!smack)
                         mac_smack_apply(devnode, NULL);
         }
@@ -340,7 +340,7 @@ void udev_node_add(struct udev_device *dev, bool apply,
         char filename[UTIL_PATH_SIZE];
         struct udev_list_entry *list_entry;
 
-        log_debug("handling device node '%s', devnum=%s, mode=%#o, uid=%d, gid=%d",
+        log_debug("handling device node '%s', devnum=%s, mode=%#o, uid="UID_FMT", gid="GID_FMT,
                   udev_device_get_devnode(dev), udev_device_get_id_filename(dev), mode, uid, gid);
 
         if (node_permissions_apply(dev, apply, mode, uid, gid, seclabel_list) < 0)
