@@ -1277,6 +1277,18 @@ int main(int argc, char *argv[]) {
 
         udev_list_node_init(&event_list);
 
+        if (!arg_debug) {
+                int fd;
+
+                fd = open("/dev/null", O_RDWR);
+                if (fd >= 0) {
+                        dup2(fd, STDIN_FILENO);
+                        dup2(fd, STDOUT_FILENO);
+                        dup2(fd, STDERR_FILENO);
+                        close(fd);
+                }
+        }
+
         fd_inotify = udev_watch_init(udev);
         if (fd_inotify < 0) {
                 r = log_error_errno(ENOMEM, "error initializing inotify");
@@ -1289,7 +1301,7 @@ int main(int argc, char *argv[]) {
                           IN_DELETE | IN_MOVE | IN_CLOSE_WRITE);
 
         if (access(UDEV_ROOT_RUN "/udev/rules.d", F_OK) < 0) {
-                mkdir_p(UDEV_ROOT_RUN "/udev/rules.d", 0755);
+                udev_mkdir_p(UDEV_ROOT_RUN "/udev/rules.d", 0755);
         }
         inotify_add_watch(fd_inotify, UDEV_ROOT_RUN "/udev/rules.d",
                           IN_DELETE | IN_MOVE | IN_CLOSE_WRITE);
